@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -329,12 +330,35 @@ extern insn_t * trampoline_entry_addr (SCHEME_OBJECT *, unsigned long);
 /* Given the address of a trampoline entry and the code for the
    trampoline to be invoked, stores the appropriate instruction
    sequence in the trampoline.  */
-extern bool store_trampoline_insns (insn_t *, byte_t);
+extern bool store_trampoline_insns (insn_t *, uint8_t);
 
 /* Give the address of a trampoline block, returns a pointer to the
    start of the trampoline's storage area.  */
 extern SCHEME_OBJECT * trampoline_storage (SCHEME_OBJECT *);
 
+typedef enum
+{
+  TRAMPOLINE_K_RETURN_TO_INTERPRETER,
+  TRAMPOLINE_K_APPLY,
+  TRAMPOLINE_K_ARITY,		/* unused */
+  TRAMPOLINE_K_ENTITY,		/* unused */
+  TRAMPOLINE_K_INTERPRETED,	/* unused */
+  TRAMPOLINE_K_LEXPR_PRIMITIVE,
+  TRAMPOLINE_K_PRIMITIVE,
+  TRAMPOLINE_K_LOOKUP,
+  TRAMPOLINE_K_1_0,
+  TRAMPOLINE_K_2_1,
+  TRAMPOLINE_K_2_0,
+  TRAMPOLINE_K_3_2,
+  TRAMPOLINE_K_3_1,
+  TRAMPOLINE_K_3_0,
+  TRAMPOLINE_K_4_3,
+  TRAMPOLINE_K_4_2,
+  TRAMPOLINE_K_4_1,
+  TRAMPOLINE_K_4_0,
+  TRAMPOLINE_K_REFLECT_TO_INTERFACE = 0x3A
+} trampoline_type_t;
+
 #ifndef UTILITY_RESULT_DEFINED
 #ifdef CMPINT_USE_STRUCS
 
@@ -426,7 +450,6 @@ extern void declare_builtin (unsigned long, const char *);
 extern utility_proc_t comutil_return_to_interpreter;
 extern utility_proc_t comutil_operator_apply_trap;
 extern utility_proc_t comutil_operator_arity_trap;
-extern utility_proc_t comutil_operator_entity_trap;
 extern utility_proc_t comutil_operator_interpreted_trap;
 extern utility_proc_t comutil_operator_lexpr_trap;
 extern utility_proc_t comutil_operator_primitive_trap;
